// Tympanus CNE slider script for Impera CMS.
(function (window, $, undefined) {

  $.Slideshow = function (options, element) {
                  this.$el = $(element);
                  this.$preloader = $('<div class="tympanus-cne-loading">Loading...</div>');

                  // images
                  this.$images = this.$el.find('div.tympanus-cne-images > img').hide();
                  this.$images.each(function () {
                                      var url = $(this).attr('data-link');
                                      if ((typeof(url) == 'string') && (jQuery.trim(url) != '')) {
                                        $(this).css({'cursor': 'pointer'}).bind('click', function (event) {
                                                                                           top.location.replace(url);
                                                                                         });
                                        
                                      }
                                    });

                  // total number of images
                  this.imgCount = this.$images.length;
                  this.isAnimating = false;
                  this._init(options);
                };

  $.Slideshow.defaults = {
    current: 0
  };

  $.Slideshow.prototype = {
    _init: function (options) {
             this.options = $.extend(true, {}, $.Slideshow.defaults, options);

             // validate options
             this._validate();
             this.current = this.options.current;
             this.$preloader.appendTo(this.$el);
             var instance = this;
             this._preloadImages(function () {
                                   instance.$preloader.hide();
                                   instance.$images.eq(instance.current).show();
                                   instance.bar = new $.NavigationBar(instance.imgCount,
                                                                      instance._getStatus());
                                   instance.bar.getElement().appendTo(instance.$el);
                                   instance._initEvents();

                                   // fix container size
                                   var $div = instance.$el.find('div.tympanus-cne-images');
                                   var w = $div.width();
                                   var h = $div.height();
                                   instance.$images.css({'width': w + 'px',
                                                         'height': h + 'px'});
                                 });
           },
    _preloadImages: function (callback) {
                      var loaded = 0, awaited = this.imgCount * 2, instance = this;
                      this.$images.each(function (i) {
                                          var $img = $(this);

                                          // large image
                                          $('<img/>').load(function () {
                                                             ++loaded;
                                                             if (loaded >= awaited) callback.call();
                                                           }).attr('src', $img.attr('src'));

                                          // thumb
                                          var url = $img.data('thumb');
                                          if (typeof(url) != 'string') url = $img.attr('data-thumb');
                                          if ((typeof(url) == 'string') && (jQuery.trim(url) != '')) {
                                            $('<img/>').load(function () {
                                                               ++loaded;
                                                               if (loaded >= awaited) callback.call();
                                                             }).attr('src', url);
                                          } else {
                                            --awaited;
                                            if (loaded >= awaited) callback.call();
                                          }
                                        });
                    },
    _validate: function () {
                 if (this.options.current < 0 || this.options.current >= this.imgCount) this.options.current = 0;
               },
    _getStatus: function () {
                  var $currentImg = this.$images.eq(this.current), $nextImg, $prevImg;
                  (this.current === 0) ? $prevImg = this.$images.eq(this.imgCount - 1)
                                       : $prevImg = $currentImg.prev();
                  (this.current === this.imgCount - 1) ? $nextImg = this.$images.eq(0)
                                                       : $nextImg = $currentImg.next();

                  // get urls
                  var pi_url = (typeof(pi_url = $prevImg.attr('data-thumb')) == 'string') ? jQuery.trim(pi_url) : '';
                  if (pi_url == '') pi_url = (typeof(pi_url = $prevImg.attr('src')) == 'string') ? jQuery.trim(pi_url) : '';
                  if (pi_url != '') pi_url = ((pi_url.indexOf('?') >= 0) ? pi_url + '&'
                                                                         : pi_url + '?') + 'unique=' + Math.random();
                  var ni_url = (typeof(ni_url = $nextImg.attr('data-thumb')) == 'string') ? jQuery.trim(ni_url) : '';
                  if (ni_url == '') ni_url = (typeof(ni_url = $nextImg.attr('src')) == 'string') ? jQuery.trim(ni_url) : '';
                  if (ni_url != '') ni_url = ((ni_url.indexOf('?') >= 0) ? ni_url + '&'
                                                                         : ni_url + '?') + 'unique=' + Math.random();

                  // get values
                  var pp_value = (typeof(pp_value = $prevImg.attr('data-price')) == 'string') ? jQuery.trim(pp_value) : '';
                  var pt_value = (typeof(pt_value = $prevImg.attr('title')) == 'string') ? jQuery.trim(pt_value) : '';
                  var cp_value = (typeof(cp_value = $currentImg.attr('data-price')) == 'string') ? jQuery.trim(cp_value) : '';
                  var ct_value = (typeof(ct_value = $currentImg.attr('title')) == 'string') ? jQuery.trim(ct_value) : '';
                  var cd_value = (typeof(cd_value = $currentImg.attr('data-description')) == 'string') ? jQuery.trim(cd_value) : '';
                  var np_value = (typeof(np_value = $nextImg.attr('data-price')) == 'string') ? jQuery.trim(np_value) : '';
                  var nt_value = (typeof(nt_value = $nextImg.attr('title')) == 'string') ? jQuery.trim(nt_value) : '';

                  // result
                  return {
                    prevSource: pi_url,
                    nextSource: ni_url,
                    prevPrice: (pp_value != '') ? pp_value : '&nbsp;',
                    prevTitle: (pt_value != '') ? pt_value : '&nbsp;',
                    currentPrice: (cp_value != '') ? cp_value : '&nbsp;',
                    currentTitle: (ct_value != '') ? ct_value : '&nbsp;',
                    currentDescription: cd_value,
                    nextPrice: (np_value != '') ? np_value : '&nbsp;',
                    nextTitle: (nt_value != '') ? nt_value : '&nbsp;'
                  };
                },
    _initEvents: function () {
                   var instance = this;
                   this.bar.$navPrev.bind('click.slideshow', function (event) {
                                                               instance._navigate('prev');
                                                               return false;
                                                             });
                   this.bar.$navNext.bind('click.slideshow', function (event) {
                                                               instance._navigate('next');
                                                               return false;
                                                             });
                   this.bar.set(this._getStatus());
                 },
    _navigate: function (dir) {
                 if (this.isAnimating) return false;
                 this.isAnimating = true;
                 var $curr = this.$images.eq(this.current).css('z-index', 998),
                     instance = this;
                 (dir === 'prev') ? (this.current === 0) ? this.current = this.imgCount - 1
                                                         : --this.current
                                  : (this.current === this.imgCount - 1) ? this.current = 0
                                                                         : ++this.current;
                 this.$images.eq(this.current).show();
                 $curr.fadeOut(400, function () {
                                      $(this).css('z-index', 1);
                                      instance.isAnimating = false;
                                    });
                 this.bar.set(this._getStatus());
               }
  };

  $.NavigationBar = function (imgCount, status) {
                      this._init(imgCount, status);
                    };

  $.NavigationBar.prototype = {
    _init: function (imgCount, status) {
             this.$el = $('#tympanus-cne-slider-template').tmpl(status);

             // navigation
             this.$navPrev = this.$el.find('a.tympanus-cne-nav-prev');
             this.$thumbPrev = this.$navPrev.children('div').children('img');
             this.$navNext = this.$el.find('a.tympanus-cne-nav-next');
             this.$thumbNext = this.$navNext.children('div').children('img');

             // navigation status
             this.$statusPrev = this.$el.find('div.tympanus-cne-nav-content-prev > h3.tympanus-cne-nav-content-title');
             this.$pricePrev = this.$el.find('div.tympanus-cne-nav-content-prev > span.tympanus-cne-nav-content-price');
             this.$statusCurrent = this.$el.find('div.tympanus-cne-nav-content-current > h2.tympanus-cne-nav-content-title');
             this.$priceCurrent = this.$el.find('div.tympanus-cne-nav-content-current > span.tympanus-cne-nav-content-price');
             this.$descriptionCurrent = this.$el.find('div.tympanus-cne-nav-description');
             this.$statusNext = this.$el.find('div.tympanus-cne-nav-content-next > h3.tympanus-cne-nav-content-title');
             this.$priceNext = this.$el.find('div.tympanus-cne-nav-content-next > span.tympanus-cne-nav-content-price');

             // just show current image description if only one image
             if (imgCount <= 1) {
               this.$navPrev.hide();
               this.$navNext.hide();
               this.$statusPrev.parent().hide();
               this.$pricePrev.parent().hide();
               this.$statusNext.parent().hide();
               this.$priceNext.parent().hide();
             }
           },
    getElement: function () {
                  return this.$el;
                },

    // set the current, previous and next descriptions, and also the previous and next thumbs
    set: function (status) {
           this.$thumbPrev.attr('src', status.prevSource);
           this.$thumbNext.attr('src', status.nextSource);
           this.$statusPrev.html(status.prevTitle);
           this.$pricePrev.html(status.prevPrice);
           this.$statusCurrent.html(status.currentTitle);
           this.$priceCurrent.html(status.currentPrice);
           this.$descriptionCurrent.html(status.currentDescription);
             if (status.currentDescription == '') this.$descriptionCurrent.hide();
             else this.$descriptionCurrent.show();
           this.$statusNext.html(status.nextTitle);
           this.$priceNext.html(status.nextPrice);
         }
  };

  var logError = function (message) {
                   if (this.console) {
                     console.error(message);
                   }
                 };

  $.fn.slideshow = function (options) {
                     if (typeof options === 'string') {
                       var args = Array.prototype.slice.call(arguments, 1);
                       this.each(function () {
                                   var instance = $.data(this, 'slideshow');
                                   if (!instance) {
                                     logError('cannot call methods on slideshow prior to initialization; ' +
                                              'attempted to call method: ' + options);
                                     return;
                                   }
                                   if (!$.isFunction(instance[options]) || options.charAt(0) === "_") {
                                     logError('no such method (' + options + ') for slideshow instance');
                                     return;
                                   }
                                   instance[options].apply(instance, args);
                                 });
                     } else {
                       this.each(function () {
                                   var instance = $.data(this, 'slideshow');
                                   if (!instance) {
                                     $.data(this, 'slideshow', new $.Slideshow(options, this));
                                   }
                                 });
                     }
                     return this;
                   };

})(window, jQuery);
